import { BadRequestException, NotFoundException } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { Address } from '../../database/schemas/address.schema';
import { User, UserEntity } from '../../database/schemas/user.schema';
import { EncryptorService } from '../../security/encryptor.service';
import { UserController } from './user.controller';
import { UserCreateDto } from './user.dto';
import { UserService } from './user.service';

const userDataMock: UserEntity = {
  _id: '21731837129837',
  administrador: false,
  contatoEmergencia: '51987654321',
  contatosProfissionais: [],
  cpf: '12345678910',
  email: 'teste@email.com',
  nome: 'Usuario Teste',
  password: 'abc123',
  telefone: '51987654321',
  endereco: {
    cep: '12345678',
    cidade: 'POA',
    nomeRua: 'Tarso Dutra',
    numeroCasa: '25',
    bairro: 'Petropolis',
    estado: 'RS',
  },
};

const findByIdMock = jest.fn();
const createMock = jest.fn();
const findOneMock = jest.fn();
const encryptorMock = {
  encrypt: jest.fn().mockReturnValue('senha encriptada'),
};

const userModelMock = {
  find: () => ({ exec: () => [userDataMock] }),
  findById: findByIdMock,
  create: createMock,
  findOne: findOneMock,
};

describe('UserController', () => {
  let userController: UserController;

  beforeEach(async () => {
    const app: TestingModule = await Test.createTestingModule({
      controllers: [UserController],
      providers: [
        UserService,
        {
          provide: getModelToken(User.name),
          useValue: userModelMock,
        },
        {
          provide: EncryptorService,
          useValue: encryptorMock,
        },
      ],
    }).compile();

    userController = app.get<UserController>(UserController);
  });

  describe('findAll', () => {
    it('deve retornar usuario mockado"', async () => {
      const result = await userController.findAll();
      expect(result.length).toEqual(1);
      expect(result[0]).toEqual(userDataMock);
    });
  });

  describe('findById', () => {
    it('deve retornar usuario mockado"', async () => {
      findByIdMock.mockReturnValue(userDataMock);
      const result = await userController.findById(userDataMock._id);
      expect(result).toEqual(userDataMock);
    });

    it('deve retornar 404 quando id inválido"', async () => {
      findByIdMock.mockRejectedValue(null);

      expect(userController.findById(userDataMock._id)).rejects.toThrowError(
        NotFoundException,
      );
    });
  });

  describe('create', () => {
    it('deve criar usuário"', async () => {
      const dto: UserCreateDto = {
        cep: userDataMock.endereco.cep,
        contatoEmergencia: userDataMock.contatoEmergencia,
        cpf: userDataMock.cpf,
        email: userDataMock.email,
        password: userDataMock.password,
        nome: userDataMock.nome,
        telefone: userDataMock.telefone,
        numeroINSS: userDataMock.numeroINSS,
      };
      const hash = 'senha encriptada';
      await userController.create(dto);
      const createModel = { ...userDataMock,
        endereco: { cep: dto.cep } as Address,
        password: hash
      };
      delete createModel._id;
      expect(createMock).toHaveBeenCalledWith(createModel);
    });

    it('quando alguma informação já existir não deve criar usuário"', async () => {
      const dto: UserCreateDto = {
        cep: userDataMock.endereco.cep,
        contatoEmergencia: userDataMock.contatoEmergencia,
        cpf: userDataMock.cpf,
        email: userDataMock.email,
        password: userDataMock.password,
        nome: userDataMock.nome,
        telefone: userDataMock.telefone,
        numeroINSS: userDataMock.numeroINSS,
      };

      findOneMock.mockReturnValue(userDataMock);

      expect(userController.create(dto)).rejects.toThrowError(
        BadRequestException,
      );
    });
  });
});
